'===============================================================================
'               B&L Service
'               homepage : www.b-und-l-service.de
'               email    : info@www.b-und-l-service
'               J. Beushausen
'===============================================================================
' 07.08.2015    Demoprogramm Server fr die serielle Schnittstelle
'===============================================================================

'hardware init - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

$regfile = "m32def.dat"                        ' used processor
$crystal = 14745600                             ' used crystal frequency for processor
baud = 9600                                     ' baudrate

on urxc Com1_Int                                ' assign IRQ service
enable urxc
enable interrupts
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

declare sub ResetParameter
declare sub LoadParameterFromEEPROM
declare sub SaveParameterToEEPROM

declare function isValidValueChar(byref aChar as  String) as  Byte
declare function isValidIndexChar(byval aChar as  String) as  Byte
declare function ParseRXStr(byval strRX as string, byref index as byte, byref sValue as single, byref bCommand as byte) as integer

declare function GetTempRaw() as  Byte
declare function CalculateTemp(byval bTemp as  Byte, sOffset as  Single, sSlope as Single) as  Single
declare function WasteTime() as  Byte
declare sub SendToPC(byval sTemp as single)

const cmdNoCommand   = 255
const cmdRX          = 1
const cmdTX          = 2
const cmdSave        = 3
const cmdSend        = 4
const cmdSendVersion = 5
const cmdReset       = 6

const CR = 13
const LF = 10

const paOK    = 0
const paError = 1

const parOffset = 15        '  [O]ffset ASCII 79 - 64
const parSlope  = 19        '  [S]lope  ASCII 83 - 64
const parTempSimulation = 20'  [T]emp   ASCII 84 - 64

const maxParam = 25

dim s_ParamFld(maxParam) as  Single         ' RAM
dim sE_ParamFld(maxParam) as  ERAM Single  ' EEPROM

dim strVersion as  String * 20
dim strRX as  String * 20

dim b_Udr1 as  Byte                         ' buffer for received character
dim b_ParserResult as Byte
dim b_Command as  Byte
dim s_Value as  Single
dim b_Index as  Byte
dim i_ParseResult as integer

dim b_TempRaw as  Byte
dim b_TempSimulation as  Byte
dim b_upDwn as  Byte
dim s_Temp as  Single
dim b_Break as Byte
dim b_RXFlag as  Byte


  strVersion = " V 1.0"
  print "Server Test "; strVersion     ' print version on start of program

  call LoadParameterFromEEPROM

  strRX = ""
  b_TempSimulation = 0
  b_UpDwn = 1
  b_Command = cmdNoCommand

  do
     if b_RXFlag <> 0 then
       'LF received, now parse received string
       'print "  RX ";strRX 'DEBUG

       i_ParseResult = ParseRXStr(strRX, b_Index, s_Value, b_Command)
       strRX = ""    ' clear receive string
       b_RXFlag = 0  ' clear flag

       if i_ParseResult = paOk  then
           b_RXFlag = 0  ' clear receive Flag

           ' process commands
          select case b_command
            case cmdRX          :  s_ParamFld(b_Index) = s_Value
                                   print "ok"
                                    b_Command = cmdNoCommand

            case cmdTX          :   print s_ParamFld(b_Index)
                                    b_Command = cmdNoCommand

            case cmdSendVersion : print strVersion
                                    b_Command = cmdNoCommand

            case cmdReset       : call ResetParameter
                                   print "ok"
                                    b_Command = cmdNoCommand

            case cmdSave        : call SaveParameterToEEPROM
                                   print "ok"
                                    b_Command = cmdNoCommand

           end SELECT

        else
          print "error"
        end IF
     end IF

     if s_ParamFld(parTempSimulation) > 0 then
       b_TempRaw = GetTempRaw()   ' simulated ADC 8 Bit
       s_Temp = CalculateTemp(b_TempRaw, s_ParamFld(parOffset), s_ParamFld(parSlope))
       call SendToPC(s_Temp)
     end IF

     waitms 100
     b_Break = WasteTime()
  loop           'endless main loop

end 'end program

'-------------------------------------------------------------------------------
' = = = = = = = = = = = = = subroutines and functions   = = = = = = = = = = = =
'-------------------------------------------------------------------------------

sub ResetParameter
local _b as  Byte
  ' clear parameter field
  for _b = 1 to maxParam
    s_ParamFld(_b) = 0
    sE_ParamFld(_b) = 0
  next _b

 ' default values
 s_ParamFld(parOffset) = -20
 sE_ParamFld(parOffset) = -20

 s_ParamFld(parSlope) = 0.549
 sE_ParamFld(parSlope) = 0.549
end SUB
'-------------------------------------------------------------------------------

sub LoadParameterFromEEPROM
local _b as  Byte
  ' load parameter field from EEPROM
  for _b = 1 to maxParam
    s_ParamFld(_b) = sE_ParamFld(_b)
  next _b
end SUB
'-------------------------------------------------------------------------------

sub SaveParameterToEEPROM
local _b as  Byte
  ' save parameter field to EEPROM
  for _b = 1 to maxParam
    sE_ParamFld(_b) = s_ParamFld(_b)
  next _b
end SUB
'-------------------------------------------------------------------------------

function GetTempRaw() as  Byte
local _iStep as  Integer
  ' simulate 8 bit temerature ADC

  if b_TempSimulation >= 251 then
    _iStep = -2   ' slow
  end IF

  if b_TempSimulation <= 0 then
      _iStep = 4 ' faster
  end IF

  b_TempSimulation = b_TempSimulation + _iStep

  GetTempRaw = b_TempSimulation
end FUNCTION
'-------------------------------------------------------------------------------

function CalculateTemp(byval bTemp as  Byte, sOffset as  Single, sSlope as Single) as  Single
local _sHelp as  Single
   'calulate temperature vith offset and slope
   _sHelp = bTemp*sSlope
   CalculateTemp = sOffset + _sHelp
end FUNCTION
'-------------------------------------------------------------------------------

function WasteTime() as  Byte
local _i as  Integer
  ' waste time, simulation of long calculation
  WasteTime = 0
  for _i = 1 to 1000
    waitms 1
    if b_RXFlag <> 0 then
      WasteTime = 1
      exit FUNCTION
    end IF
  next _i

end function
'-------------------------------------------------------------------------------

sub SendToPC(byval sTemp as single)
  print "temp ";fusing(sTemp, "#.##")
end SUB
'-------------------------------------------------------------------------------


function isValidValueChar(byref aChar as  String) as  Byte
local _valid as  Byte
local _asc as  Byte

  _valid = 1

  if aChar = " " then   ' no blanks
    aChar = ""
  end IF

  if aChar = "," then   ' no comma as decimal separator
    aChar = "."
  end IF

  if aChar = "+" then   ' no plus sign
    aChar = ""
  end IF

  _asc = ASC(aChar)

  if aChar <> "" then
    if _asc < ASC("0") or _asc > ASC("9") then
      if _asc <> 46 and _asc <> ASC("-") and _asc <> ASC(";") and _asc <> ASC("e") and _asc <> ASC("E") then
         _valid = 0
      end IF
    end IF
  end IF

  isValidValueChar = _valid
end FUNCTION
'-------------------------------------------------------------------------------

function isValidIndexChar(byval aChar as  String) as  Byte
local _valid as  Byte
local _asc as  Byte

  _valid = 1
  _asc = ASC(aChar)

  if _asc >= ASC("a") and _asc <= ASC("z") or _asc >= ASC("A") and _asc <= ASC("Z") then
    _valid = 1
  else
    if _asc = ASC("!") or _asc >= ASC("?") or _asc <= ASC("$") then
      _valid = 1
    end IF
  end IF

  isValidIndexChar = _valid
end FUNCTION
'-------------------------------------------------------------------------------

function ParseRXStr(byval strRX as string, byref index as byte, byref sValue as single, byref bCommand as byte) as integer
local strVal   as  String * 20
local strIndex as  String * 20
local length as  Integer : local i as  Integer
local hasSemicolon as  Byte : local _index as  Integer
local char1 as String * 1 : local strI as string*1

  ' parse received string
  ParseRXStr = paOK
  hasSemicolon = 0
  length = len(strRX)

  strVal = ""
  strIndex = ""

  strRX = trim(strRX)

  for i = 1 to length
    char1 = mid(strRX, i, 1)

    if hasSemicolon = 0 then
      if isValidIndexChar(char1) = 1 then
        strIndex = strIndex + char1
      end IF
      if char1 = ";" then
        incr hasSemicolon
      end IF
    else
      if hasSemicolon = 1 then
         if isValidValueChar(Char1) = 1 then
           strVal = strVal + char1
         else
           ParseRXStr = ParseRXStr or paError
         end IF
       end IF
    end IF
  next i

  if strIndex <> "" then
     strI = mid(strRX, 1, 1)
     _index = ASC(strI)

     ' lower case charcter
     if _index >= 97 and _index <= 122 then  ' a .. z
       index = _index - 96       ' 1 .. 25
       bCommand  = cmdTX
     end IF

     ' upper case charcter
     if _index >= 65 and _index <= 90 then  ' A .. Z
       index = _index - 64    ' 1 ,, 25
       bCommand  = cmdRX
     end IF

   ' special command character
     if _index = 33 or _index = 36 or _index <= 63 then     ' !, $ oder ?
        select case _index
          case 33  : bCommand  = cmdReset        ' !
          case 36  : bCommand  = cmdSave         ' $
          case 63  : bCommand  = cmdSendVersion  ' ?
          case else  bCommand  = cmdNoCommand
        end SELECT
     end IF
  else
    ParserxStr = ParserxStr or paError
  end IF

  if ParseRXStr = paOK then
    if strVal <> "" then
      sValue = val(strVal)
    else
      if hasSemicolon = 1 then
         ParseRXStr = paError
      end IF
    end IF

    if bCommand = cmdRX and strVal = "" then
      ParseRXStr = paError
    end IF
  end IF

end FUNCTION

'-------------------------------------------------------------------------------
' = = = = = = = = = = = = = = = IRQ Service = = = = = = = = = = = = = = = = = =
'-------------------------------------------------------------------------------
Com1_Int:
  b_udr1 = udr
  ' add received character to receive string
  if b_udr1 <> CR and b_udr1 <> LF then
    strRX = strRX + chr(b_udr1)
  end IF

  if b_Udr1 = CR then
    b_RXFlag = 1
  end IF
return
'-------------------------------------------------------------------------------